<?php
/**
 * Bitpay Gateway for Easy Digital Downloads
 *
 * @author                HANNANStd
 * @package            Iranian_Gateways
 * @subpackage            Gateways
 */
if (!defined('ABSPATH')) exit;

if (!class_exists('EDD_Bitpay_Gateway')) :

    /**
     * Bitpay Gateway for Easy Digital Downloads
     *
     * @author                HANNANStd
     * @package            Iranian_Gateways
     * @subpackage            Gateways
     */
    class EDD_Bitpay_Gateway
    {
        /**
         * Gateway keyname
         *
         * @var                string
         */
        public $keyname;

        /**
         * Initialize gateway and hook
         *
         * @return                void
         */
        public function __construct()
        {
            $this->keyname = 'bitpay';

            add_filter('edd_payment_gateways', array($this, 'add'));
            add_action($this->format('edd_{key}_cc_form'), array($this, 'cc_form'));
            add_action($this->format('edd_gateway_{key}'), array($this, 'process'));
            add_action($this->format('edd_verify_{key}'), array($this, 'verify'));
            add_filter('edd_settings_gateways', array($this, 'settings'));
            add_filter('edd_settings_sections_gateways', array($this, 'section'));
            add_action('edd_payment_receipt_after', array($this, 'receipt'));
            add_action('init', array($this, 'listen'));
        }

        /**
         * Add gateway to list
         *
         * @param                array $gateways Gateways array
         * @return                array
         */
        public function add($gateways)
        {
            global $edd_options;

            $gateways[$this->keyname] = array(
                'checkout_label' => isset($edd_options['bitpay_label']) ? $edd_options['bitpay_label'] : 'پرداخت آنلاین بیت پی',
                'admin_label' => 'بیت پی'
            );

            return $gateways;
        }

        /**
         * CC Form
         * We don't need it anyway.
         *
         * @return                bool
         */
        public function cc_form()
        {
        }

        /**
         * Process the payment
         *
         * @param                array $purchase_data
         * @return                void
         */
        public function process($purchase_data)
        {
            global $edd_options, $current_user;
            $payment = $this->insert_payment($purchase_data);
            $CustomerId = ($current_user && $user_data = get_userdata($current_user->ID)) ? $current_user->ID : 0;

            if ($payment) {

                //storing data
                @session_start();
                $_SESSION['Bitpay_payment'] = $payment;
                EDD()->session->set('Bitpay_payment_edd', $payment);
                set_transient('Bitpay_payment_transient', $payment, 60 * 60 * 12);

                $CallbackURL = add_query_arg(array('verify_edd_' . $this->keyname => '1', 'pid' => $payment), esc_url(home_url('/')));
				$CallbackURL = urlencode($CallbackURL);
				
                if (edd_get_currency() == 'IRT' || $edd_options['currency'] == 'IRT') {
                    $Amount = $purchase_data['price'] * 10;
                } else {
                    $Amount = $purchase_data['price'];
                }
                $API = $edd_options[$this->keyname . '_api'];

				$products = array();
				$cart_items = $purchase_data['cart_details'];
				foreach ( (array) $cart_items as $item ) {
					$products[] = $item['name'] . ' ';
				}
				$products = implode( ' - ', $products );
			
				$Description = 'پرداخت شماره #' . $payment  . ' | محصولات : ' . $products;
			
				$first_name = isset ( $purchase_data['user_info']['first_name'] ) ? $purchase_data['user_info']['first_name'] : '';
				$last_name = isset ( $purchase_data['user_info']['last_name'] ) ? $purchase_data['user_info']['last_name'] : '';
				$FullName = $first_name . ' ' . $last_name;
				$Email = $purchase_data['user_email'];

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, 'https://bitpay.ir/payment/gateway-send');
				curl_setopt($ch, CURLOPT_POSTFIELDS, "api=$API&amount=$Amount&redirect=$CallbackURL&name=$FullName&email=$Email&description=$Description&factorId=$payment&reagentId=287969");
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
				$result = curl_exec($ch);
				
				if (!empty($result) && is_numeric($result) && $result > 0) {

					$Payment_URL = sprintf('https://bitpay.ir/payment/gateway-%s', $result);
					
					if ( ! headers_sent() ) { 
						header('Location: ' . $Payment_URL ); 
						exit;
					}
					else {
						echo "<script type='text/javascript'>window.onload = function () { top.location.href = '" . $Payment_URL . "'; };</script>";
						exit;
					}

                } else {

                    $fault = $result."1";

                    edd_insert_payment_note($payment, 'در برقراری ارتباط با درگاه مشکلی پیش آمد. کد خطا : ' . $fault);
                    edd_insert_payment_note($payment, 'علت خطا : ' . $this->error_reason($fault));
                    edd_update_payment_status($payment, 'failed');

                    edd_set_error('bitpay_connection_error', 'در برقراری ارتباط با درگاه مشکلی پیش آمد. علت خطا : ' . $this->error_reason($fault));
                    edd_send_back_to_checkout();
                }


            } else {
                edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
            }
        }

        /**
         * Verify the payment
         *
         * @return                void
         */
        public function verify()
        {
            global $edd_options;

            $session = edd_get_purchase_session();
            $payment = edd_get_purchase_id_by_key($session['purchase_key']);
		
			@session_start();
            if (isset($_GET['pid']) && is_numeric($_GET['pid']))
                $payment_id = $_GET['pid'];
            elseif (EDD()->session->get('Bitpay_payment_edd'))
                $payment_id = EDD()->session->get('Bitpay_payment_edd');
            elseif (!empty($payment) && is_numeric($payment))
                $payment_id = $payment;
            elseif (!empty($_SESSION['Bitpay_payment']) && is_numeric($_SESSION['Bitpay_payment']))
                $payment_id = $_SESSION['Bitpay_payment'];
            else
                $payment_id = get_transient('Bitpay_payment_transient');
            delete_transient('Bitpay_payment_transient');

            if (!$payment_id) {
                wp_die('رکورد پرداخت موردنظر وجود ندارد!');
            }
			
            $Amount = edd_get_payment_amount($payment_id);
            if (edd_get_currency() == 'IRT' || $edd_options['currency'] == 'IRT') {
                $Amount = $Amount * 10;
            }
            $Api = $edd_options[$this->keyname . '_api'];

            $transaction_id = $trans_id = isset($_POST['trans_id']) ?  sanitize_text_field($_POST['trans_id']) : '';
            $id_get = isset($_POST['id_get']) ?  sanitize_text_field($_POST['id_get']) : '';
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, 'https://bitpay.ir/payment/gateway-result-second');
			curl_setopt($ch, CURLOPT_POSTFIELDS, "api=$Api&id_get=$id_get&trans_id=$trans_id");
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
			$result = curl_exec($ch);
			curl_close($ch);
			
			if (!empty($result) && $result == 1) {
				$status = 'completed';
				$fault = 0;
			}
			else {
				$status = 'failed';
				$fault = $result;
			}
					
            if (!empty($transaction_id)) {
				
                edd_insert_payment_note($payment_id, 'شماره تراکنش: ' . $transaction_id . ' | Id_get: ' . $id_get);
				
                if (function_exists('edd_set_payment_transaction_id')) {
                    edd_set_payment_transaction_id($payment_id, $transaction_id);
                }
				
				if (function_exists('edd_update_payment_meta')) {
					edd_update_payment_meta( $payment_id, 'bitpay_transaction_id', $transaction_id );
				}
            }


            if ($status == 'completed') {

                edd_empty_cart();

                edd_insert_payment_note($payment_id, 'پرداخت موفقیت آمیز بود.');
                edd_update_payment_status($payment_id, 'publish');
                edd_send_to_success_page();
                exit;
            } else {
                edd_update_payment_status($payment_id, 'failed');
                edd_insert_payment_note($payment_id, 'در تایید پرداخت خطایی رخ داده است . کد خطا : ' . $fault);
                edd_insert_payment_note($payment_id, 'علت خطا : ' . $this->error_reason($fault));
                wp_redirect(get_permalink($edd_options['failure_page']));
                exit;
            }

        }


        /**
         * Gateway settings section
         *
         * @param                array $sections
         * @return                array
         */
        public function section($sections)
        {
            $sections[$this->keyname . '_checkout'] = 'بیت پی';
            return $sections;
        }

        /**
         * Gateway settings
         *
         * @param                array $settings
         * @return                array
         */
        public function settings($settings)
        {
            $gateway_settings = array(
                $this->keyname . '_header' => array(
                    'id' => $this->keyname . '_header',
                    'type' => 'header',
                    'name' => '<strong>درگاه بیت پی</strong>'
                ),
                $this->keyname . '_api' => array(
                    'id' => $this->keyname . '_api',
                    'name' => 'API',
                    'type' => 'text',
                    'size' => 'regular'
                ),
                $this->keyname . '_label' => array(
                    'id' => $this->keyname . '_label',
                    'name' => 'نام درگاه در صفحه پرداخت',
                    'type' => 'text',
                    'size' => 'regular',
                    'std' => 'درگاه پرداخت آنلاین بیت پی'
                ),
            );

            if (version_compare(EDD_VERSION, 2.5, '>=')) {
                $gateway_settings = array($this->keyname . '_checkout' => $gateway_settings);
            }

            return array_merge($settings, $gateway_settings);
        }

        /**
         * Receipt field for payment
         *
         * @param                object $payment
         * @return                void
         */
        public function receipt($payment)
        {
            $bitpay_transaction_id = edd_get_payment_meta($payment->ID, 'bitpay_transaction_id');

            if ($bitpay_transaction_id) {
                echo '<tr class="bitpay-id-get-row iranian-gateway-field"><td><strong>شماره تراکنش:</strong></td><td>' . $bitpay_transaction_id . '</td></tr>';
            }
        }

        /**
         * Format a string, replaces {key} with $keyname
         *
         * @param            string $string To format
         * @return            string Formatted
         */
        private function format($string)
        {
            return str_replace('{key}', $this->keyname, $string);
        }

        /**
         * Inserts a payment into database
         *
         * @param            array $purchase_data
         * @return            int $payment_id
         */
        private function insert_payment($purchase_data)
        {
            global $edd_options;

            $payment_data = array(
                'price' => $purchase_data['price'],
                'date' => $purchase_data['date'],
                'user_email' => $purchase_data['user_email'],
                'purchase_key' => $purchase_data['purchase_key'],
                'currency' => $edd_options['currency'],
                'downloads' => $purchase_data['downloads'],
                'user_info' => $purchase_data['user_info'],
                'cart_details' => $purchase_data['cart_details'],
                'status' => 'pending'
            );

            // record the pending payment
            $payment = edd_insert_payment($payment_data);

            return $payment;
        }

        /**
         * Listen to incoming queries
         *
         * @return            void
         */
        public function listen()
        {
            if (isset($_GET['verify_edd_' . $this->keyname]) && $_GET['verify_edd_' . $this->keyname]) {
                do_action('edd_verify_' . $this->keyname);
            }
        }

        /**
         * Error reason for Bitpay
         *
         * @param            int $error_id
         * @return            string
         */
        public function error_reason($error_id)
        {
            $message = 'خطای ناشناخته';

            switch ($error_id) {
			
				case '-11' :
					$message = 'Api ارسالی با نوع Api تعریف شده در درگاه سازگار نیست .';
                    break;
				case '-21' :
					$message = 'مقدار مبلغ نباید کمتر از 1000 ریال باشد .';
                    break;
				case '-31' :
					$message = 'آدرس بازگشت از بانک معتبر نیست .';
                    break;
				case '-41' :
					$message = 'درگاهی با اطلاعات ارسالی شما یافت نشده و یا در حالت انتظار می باشد .';
                    break;	
					
				case '-1' :
					$message = 'Api ارسالی با نوع Api تعریف شده در درگاه سازگار نیست .';
                    break;
				case '-2' :
					$message = 'trans_id ارسال شده معتبر نمی باشد .';
                    break;
				case '-3' :
					$message = 'id_get ارسال شده معتبر نمی باشد .';
                    break;
				case '-4' :
					$message = 'چنین تراکنشی یا در سیستم وجود ندارد یا ناموفق بوده است .';
                    break;

            }

            return $message;
        }
    }

endif;

new EDD_Bitpay_Gateway;